searchData={"items":[{"type":"module","title":"Styler","doc":"Styler is a formatter plugin with stronger opinions on code organization, multi-line defs and other code-style matters.","ref":"Styler.html"},{"type":"module","title":"Styler.AliasEnv","doc":"A datastructure for maintaining something like compiler alias state when traversing AST.\n\nNot anywhere as correct as what the compiler gives us, but close enough for open source work.\n\nA alias env is a map from an alias's `as` to its resolution in a context.\n\nGiven the ast for\n\n    alias Foo.Bar\n\nwe'd create the env:\n\n    %{:Bar => [:Foo, :Bar]}","ref":"Styler.AliasEnv.html"},{"type":"function","title":"Styler.AliasEnv.define/2","doc":"","ref":"Styler.AliasEnv.html#define/2"},{"type":"function","title":"Styler.AliasEnv.expand/2","doc":"","ref":"Styler.AliasEnv.html#expand/2"},{"type":"behaviour","title":"Styler.Style","doc":"A Style takes AST and returns a transformed version of that AST.\n\nBecause these transformations involve traversing trees (the \"T\" in \"AST\"), we wrap the AST in a structure\ncalled a Zipper to facilitate walking the trees.","ref":"Styler.Style.html"},{"type":"function","title":"Styler.Style.displace_comments/2","doc":"Set the line of all comments with `line` in `range_start..range_end` to instead have line `range_start`","ref":"Styler.Style.html#displace_comments/2"},{"type":"function","title":"Styler.Style.ensure_block_parent/1","doc":"Returns the current node (wrapped in a `__block__` if necessary) if it's a valid place to insert additional nodes","ref":"Styler.Style.html#ensure_block_parent/1"},{"type":"function","title":"Styler.Style.find_nearest_block/1","doc":"Returns a zipper focused on the nearest node where additional nodes can be inserted (a \"block\").\n\nThe nearest node is either the current node, an ancestor, or one of those two but wrapped in a new `:__block__` node.","ref":"Styler.Style.html#find_nearest_block/1"},{"type":"function","title":"Styler.Style.fix_line_numbers/2","doc":"\"Fixes\" the line numbers of nodes who have had their orders changed via sorting or other methods.\nThis \"fix\" simply ensures that comments don't get wrecked as part of us moving AST nodes willy-nilly.\n\nThe fix is rather naive, and simply enforces the following property on the code:\nA given node must have a line number less than the following node.\nEt voila! Comments behave much better.","ref":"Styler.Style.html#fix_line_numbers/2"},{"type":"function","title":"In Detail - Styler.Style.fix_line_numbers/2","doc":"For example, given document\n\n  1: defmodule ...\n  2: alias B\n  3: # this is foo\n  4: def foo ...\n  5: alias A\n\nSorting aliases the ast node for  would put `alias A` (line 5) before `alias B` (line 2).\n\n  1: defmodule ...\n  5: alias A\n  2: alias B\n  3: # this is foo\n  4: def foo ...\n\nElixir's document algebra would then encounter `line: 5` and immediately dump all comments with `line <= 5`,\nmeaning after running through the formatter we'd end up with\n\n  1: defmodule\n  2: # hi\n  3: # this is foo\n  4: alias A\n  5: alias B\n  6:\n  7: def foo ...\n\nThis function fixes that by seeing that `alias A` has a higher line number than its following sibling `alias B` and so\nupdates `alias A`'s line to be preceding `alias B`'s line.\n\nRunning the results of this function through the formatter now no longer dumps the comments prematurely\n\n  1: defmodule ...\n  2: alias A\n  3: alias B\n  4: # this is foo\n  5: def foo ...","ref":"Styler.Style.html#fix_line_numbers/2-in-detail"},{"type":"function","title":"Styler.Style.max_line/1","doc":"","ref":"Styler.Style.html#max_line/1"},{"type":"function","title":"Styler.Style.reset_newlines/1","doc":"Takes a list of nodes and clumps them up, setting `end_of_expression: [newlines: x]` to 1 for all but the final node,\nwhich gets 2 instead, (hopefully!) creating an empty line before whatever follows.","ref":"Styler.Style.html#reset_newlines/1"},{"type":"function","title":"Styler.Style.reset_newlines/2","doc":"","ref":"Styler.Style.html#reset_newlines/2"},{"type":"callback","title":"Styler.Style.run/2","doc":"`run` will be used with `Zipper.traverse_while/3`, meaning it will be executed on every node of the AST.\n\nYou can skip traversing parts of the tree by returning a Zipper that's further along in the traversal, for example\nby calling `Zipper.skip(zipper)` to skip an entire subtree you know is of no interest to your Style.","ref":"Styler.Style.html#c:run/2"},{"type":"function","title":"Styler.Style.set_line/3","doc":"Recursively sets `:line` meta to `line`. Deletes `:newlines` unless `delete_lines: false` is passed","ref":"Styler.Style.html#set_line/3"},{"type":"function","title":"Styler.Style.shift_comments/2","doc":"Perform a series of shifts in a single pass.\n\nWhen shifting comments from block A to block B, naively using two passes of `shift_comments/3` would result\nin all comments ending up in either region A or region B (because A would move to B, then all B back to A)\nThis function exists to make sure that a comment is only moved once during the swap.","ref":"Styler.Style.html#shift_comments/2"},{"type":"function","title":"Styler.Style.shift_comments/3","doc":"Change the `line` of all comments with `line` in `range` by adding `delta` to it.\nA positive delta will move the lines further down a file, while a negative delta will move them up.","ref":"Styler.Style.html#shift_comments/3"},{"type":"function","title":"Styler.Style.shift_line/2","doc":"Recursively updates `:line` meta by adding `delta`","ref":"Styler.Style.html#shift_line/2"},{"type":"function","title":"Styler.Style.update_all_meta/2","doc":"Traverses an ast node, updating all nodes' meta with `meta_fun`","ref":"Styler.Style.html#update_all_meta/2"},{"type":"function","title":"Styler.Style.without_meta/1","doc":"","ref":"Styler.Style.html#without_meta/1"},{"type":"type","title":"Styler.Style.context/0","doc":"","ref":"Styler.Style.html#t:context/0"},{"type":"module","title":"Styler.Style.Blocks","doc":"Simple 1-1 rewrites all crammed into one module to make for more efficient traversals\n\nCredo Rules addressed:\n\n* Credo.Check.Consistency.ParameterPatternMatching\n* Credo.Check.Readability.LargeNumbers\n* Credo.Check.Readability.ParenthesesOnZeroArityDefs\n* Credo.Check.Readability.PreferImplicitTry\n* Credo.Check.Readability.WithSingleClause\n* Credo.Check.Refactor.CaseTrivialMatches\n* Credo.Check.Refactor.CondStatements\n* Credo.Check.Refactor.RedundantWithClauseResult\n* Credo.Check.Refactor.WithClauses","ref":"Styler.Style.Blocks.html"},{"type":"function","title":"Styler.Style.Blocks.run/2","doc":"","ref":"Styler.Style.Blocks.html#run/2"},{"type":"module","title":"Styler.Style.Configs","doc":"Orders `Config.config/2,3` stanzas in configuration files.\n\n- ordering is done only within immediate-sibling config statements\n- assignments are moved above the configuration blocks\n- any non `config/2,3` or assignment (`=/2`) calls mark the end of a sorting block.\n  this is support having conditional blocks (`if/case/cond`) and `import_config` stanzas between blocks","ref":"Styler.Style.Configs.html"},{"type":"module","title":"Breakages - Styler.Style.Configs","doc":"If you configure the same values multiple times, Styler may swap their orders\n\n**Before**\n\n  line 04: config :foo, bar: :zab\n  line 40: config :foo, bar: :baz\n\n  # Application.fetch_env!(:foo)[:bar] => :baz\n\n**After**\n\n  line 04: config :foo, bar: :baz\n  line 05: config :foo, bar: :zab\n\n  # Application.fetch_env!(:foo)[:bar] => :zab\n\n**Fix**\n\nThe reason Styler sorts configuration is to help you noticed these duplicated configuration stanzas.\nDelete the duplicative/erroneous stanza and life will be good.","ref":"Styler.Style.Configs.html#module-breakages"},{"type":"function","title":"Styler.Style.Configs.run/2","doc":"","ref":"Styler.Style.Configs.html#run/2"},{"type":"function","title":"Styler.Style.Configs.set_lines/5","doc":"","ref":"Styler.Style.Configs.html#set_lines/5"},{"type":"module","title":"Styler.Style.Defs","doc":"Styles function heads so that they're as small as possible.\n\nThe goal is that a function head fits on a single line.\n\nThis isn't a Credo issue, and the formatter is fine with either approach. But Styler has opinions!\n\nEx:\n\nThis long declaration\n\n    def foo(%{\n      bar: baz\n    }) do\n      ...\n    end\n\nBecomes\n\n    def foo(%{bar: baz}) do\n      ...\n    end","ref":"Styler.Style.Defs.html"},{"type":"function","title":"Styler.Style.Defs.run/2","doc":"","ref":"Styler.Style.Defs.html#run/2"},{"type":"module","title":"Styler.Style.Deprecations","doc":"Transformations to soft or hard deprecations introduced on newer Elixir releases","ref":"Styler.Style.Deprecations.html"},{"type":"function","title":"Styler.Style.Deprecations.run/2","doc":"","ref":"Styler.Style.Deprecations.html#run/2"},{"type":"module","title":"Styler.Style.ModuleDirectives","doc":"Styles up module directives!\n\nThis Style will expand multi-aliases/requires/imports/use and sort the directive within its groups (except `use`s, which cannot be sorted)\nIt also adds a blank line after each directive group.","ref":"Styler.Style.ModuleDirectives.html"},{"type":"module","title":"Credo rules - Styler.Style.ModuleDirectives","doc":"Rewrites for the following Credo rules:\n\n  * `Credo.Check.Consistency.MultiAliasImportRequireUse` (force expansion)\n  * `Credo.Check.Readability.AliasOrder` (we sort `__MODULE__`, which credo doesn't)\n  * `Credo.Check.Readability.ModuleDoc` (adds `@moduledoc false` if missing. includes `*.exs` files)\n  * `Credo.Check.Readability.MultiAlias`\n  * `Credo.Check.Readability.StrictModuleLayout` (see section below for details)\n  * `Credo.Check.Readability.UnnecessaryAliasExpansion`\n  * `Credo.Check.Design.AliasUsage`","ref":"Styler.Style.ModuleDirectives.html#module-credo-rules"},{"type":"module","title":"Strict Layout - Styler.Style.ModuleDirectives","doc":"Modules directives are sorted into the following order:\n\n  * `@shortdoc`\n  * `@moduledoc`\n  * `@behaviour`\n  * `use`\n  * `import`\n  * `alias`\n  * `require`\n  * everything else (unchanged)","ref":"Styler.Style.ModuleDirectives.html#module-strict-layout"},{"type":"function","title":"Styler.Style.ModuleDirectives.run/2","doc":"","ref":"Styler.Style.ModuleDirectives.html#run/2"},{"type":"module","title":"Styler.Style.Pipes","doc":"Styles pipes! In particular, don't make pipe chains of only one pipe, and some persnickety pipe chain start stuff.\n\nRewrites for the following Credo rules:\n\n  * Credo.Check.Readability.BlockPipe\n  * Credo.Check.Readability.OneArityFunctionInPipe\n  * Credo.Check.Readability.PipeIntoAnonymousFunctions\n  * Credo.Check.Readability.SinglePipe\n  * Credo.Check.Refactor.FilterCount\n  * Credo.Check.Refactor.MapInto\n  * Credo.Check.Refactor.MapJoin\n  * Credo.Check.Refactor.PipeChainStart, excluded_functions: [\"from\"]","ref":"Styler.Style.Pipes.html"},{"type":"function","title":"Styler.Style.Pipes.run/2","doc":"","ref":"Styler.Style.Pipes.html#run/2"},{"type":"module","title":"Styler.Style.SingleNode","doc":"Simple 1-1 rewrites all crammed into one module to make for more efficient traversals\n\nCredo Rules addressed:\n\n* Credo.Check.Consistency.ParameterPatternMatching\n* Credo.Check.Readability.LargeNumbers\n* Credo.Check.Readability.ParenthesesOnZeroArityDefs\n* Credo.Check.Readability.PreferImplicitTry\n* Credo.Check.Readability.StringSigils\n* Credo.Check.Readability.WithSingleClause\n* Credo.Check.Refactor.CaseTrivialMatches\n* Credo.Check.Refactor.CondStatements\n* Credo.Check.Refactor.RedundantWithClauseResult\n* Credo.Check.Refactor.WithClauses","ref":"Styler.Style.SingleNode.html"},{"type":"function","title":"Styler.Style.SingleNode.run/2","doc":"","ref":"Styler.Style.SingleNode.html#run/2"},{"type":"module","title":"Styler.Zipper","doc":"Implements a Zipper for the Elixir AST based on Gérard Huet [Functional pearl: the\nzipper](https://www.st.cs.uni-saarland.de/edu/seminare/2005/advanced-fp/docs/huet-zipper.pdf) paper and\nClojure's `clojure.zip` API.\n\nA zipper is a data structure that represents a location in a tree from the\nperspective of the current node, also called *focus*. It is represented by a\n2-tuple where the first element is the focus and the second element is the\nmetadata/context. The metadata is `nil` when the focus is the topmost node","ref":"Styler.Zipper.html"},{"type":"function","title":"Styler.Zipper.any?/2","doc":"Traverses `zipper`, returning true when `fun.(Zipper.node(zipper))` is truthy, or false otherwise","ref":"Styler.Zipper.html#any?/2"},{"type":"function","title":"Styler.Zipper.append_child/2","doc":"Inserts the item as the rightmost child of the node at this zipper,\nwithout moving.","ref":"Styler.Zipper.html#append_child/2"},{"type":"function","title":"Styler.Zipper.children/1","doc":"Returns a list of children of the node.","ref":"Styler.Zipper.html#children/1"},{"type":"function","title":"Styler.Zipper.down/1","doc":"Returns the zipper of the leftmost child of the node at this zipper, or\nnil if no there's no children.","ref":"Styler.Zipper.html#down/1"},{"type":"function","title":"Styler.Zipper.find/3","doc":"Returns a zipper to the node that satisfies the predicate function, or `nil`\nif none is found.\n\nThe optional second parameters specifies the `direction`, defaults to\n`:next`.","ref":"Styler.Zipper.html#find/3"},{"type":"function","title":"Styler.Zipper.insert_child/2","doc":"Inserts the item as the leftmost child of the node at this zipper,\nwithout moving.","ref":"Styler.Zipper.html#insert_child/2"},{"type":"function","title":"Styler.Zipper.insert_left/2","doc":"Inserts the item as the left sibling of the node at this zipper, without\nmoving. Raises an `ArgumentError` when attempting to insert a sibling at the\ntop level.","ref":"Styler.Zipper.html#insert_left/2"},{"type":"function","title":"Styler.Zipper.insert_right/2","doc":"Inserts the item as the right sibling of the node at this zipper, without\nmoving. Raises an `ArgumentError` when attempting to insert a sibling at the\ntop level.","ref":"Styler.Zipper.html#insert_right/2"},{"type":"function","title":"Styler.Zipper.insert_siblings/2","doc":"Inserts many siblings to the right.\n\nEquivalent to\n\n    Enum.reduce(siblings, zipper, &Zipper.insert_right(&2, &1))","ref":"Styler.Zipper.html#insert_siblings/2"},{"type":"function","title":"Styler.Zipper.left/1","doc":"Returns the zipper of the left sibling of the node at this zipper, or nil.","ref":"Styler.Zipper.html#left/1"},{"type":"function","title":"Styler.Zipper.leftmost/1","doc":"Returns the leftmost sibling of the node at this zipper, or itself.","ref":"Styler.Zipper.html#leftmost/1"},{"type":"function","title":"Styler.Zipper.next/1","doc":"Returns the following zipper in depth-first pre-order.","ref":"Styler.Zipper.html#next/1"},{"type":"function","title":"Styler.Zipper.node/1","doc":"Returns the node at the zipper.","ref":"Styler.Zipper.html#node/1"},{"type":"function","title":"Styler.Zipper.prepend_siblings/2","doc":"Inserts many siblings to the left.\n\nEquivalent to\n\n    Enum.reduce(siblings, zipper, &Zipper.insert_left(&2, &1))","ref":"Styler.Zipper.html#prepend_siblings/2"},{"type":"function","title":"Styler.Zipper.prev/1","doc":"Returns the previous zipper in depth-first pre-order.\nReturns nil if the tree is already at the top.","ref":"Styler.Zipper.html#prev/1"},{"type":"function","title":"Styler.Zipper.remove/1","doc":"Removes the node at the zipper, returning the zipper that would have preceded\nit in a depth-first walk.","ref":"Styler.Zipper.html#remove/1"},{"type":"function","title":"Styler.Zipper.replace/2","doc":"Replaces the current node in the zipper with a new node.","ref":"Styler.Zipper.html#replace/2"},{"type":"function","title":"Styler.Zipper.replace_children/2","doc":"Returns the zipper with the current children of the node replaced with `children`","ref":"Styler.Zipper.html#replace_children/2"},{"type":"function","title":"Styler.Zipper.right/1","doc":"Returns the zipper of the right sibling of the node at this zipper, or nil.","ref":"Styler.Zipper.html#right/1"},{"type":"function","title":"Styler.Zipper.rightmost/1","doc":"Returns the rightmost sibling of the node at this zipper, or itself.","ref":"Styler.Zipper.html#rightmost/1"},{"type":"function","title":"Styler.Zipper.root/1","doc":"Walks the zipper all the way up and returns the root node.","ref":"Styler.Zipper.html#root/1"},{"type":"function","title":"Styler.Zipper.skip/2","doc":"Returns the zipper of the right sibling of the node at this zipper, or the\nnext zipper when no right sibling is available.\n\nThis allows to skip subtrees while traversing the siblings of a node.\n\nThe optional second parameters specifies the `direction`, defaults to\n`:next`.\n\nIf no right/left sibling is available, this function returns the same value as\n`next/1`/`prev/1`.\n\nThe function `skip/1` behaves like the `:skip` in `traverse_while/2` and\n`traverse_while/3`.","ref":"Styler.Zipper.html#skip/2"},{"type":"function","title":"Styler.Zipper.top/1","doc":"Walks the zipper all the way up and returns the top zipper.","ref":"Styler.Zipper.html#top/1"},{"type":"function","title":"Styler.Zipper.traverse/2","doc":"Traverses the tree in depth-first pre-order calling the given function for\neach node.\n\nIf the zipper is not at the top, just the subtree will be traversed.\n\nThe function must return a zipper.","ref":"Styler.Zipper.html#traverse/2"},{"type":"function","title":"Styler.Zipper.traverse/3","doc":"Traverses the tree in depth-first pre-order calling the given function for\neach node with an accumulator.\n\nIf the zipper is not at the top, just the subtree will be traversed.","ref":"Styler.Zipper.html#traverse/3"},{"type":"function","title":"Styler.Zipper.traverse_while/2","doc":"Traverses the tree in depth-first pre-order calling the given function for\neach node.\n\nThe traversing will continue if the function returns `{:cont, zipper}`,\nskipped for `{:skip, zipper}` and halted for `{:halt, zipper}`\n\nIf the zipper is not at the top, just the subtree will be traversed.\n\nThe function must return a zipper.","ref":"Styler.Zipper.html#traverse_while/2"},{"type":"function","title":"Styler.Zipper.traverse_while/3","doc":"Traverses the tree in depth-first pre-order calling the given function for\neach node with an accumulator.\n\nThe traversing will continue if the function returns `{:cont, zipper, acc}`,\nskipped for `{:skip, zipper, acc}` and halted for `{:halt, zipper, acc}`\n\nIf the zipper is not at the top, just the subtree will be traversed.","ref":"Styler.Zipper.html#traverse_while/3"},{"type":"function","title":"Styler.Zipper.up/1","doc":"Returns the zipper of the parent of the node at this zipper, or nil if at the\ntop.","ref":"Styler.Zipper.html#up/1"},{"type":"function","title":"Styler.Zipper.update/2","doc":"Replaces the current node in the zipper with the result of applying `fun` to\nthe node.","ref":"Styler.Zipper.html#update/2"},{"type":"function","title":"Styler.Zipper.zip/1","doc":"Creates a zipper from a tree node.","ref":"Styler.Zipper.html#zip/1"},{"type":"type","title":"Styler.Zipper.command/0","doc":"","ref":"Styler.Zipper.html#t:command/0"},{"type":"opaque","title":"Styler.Zipper.path/0","doc":"","ref":"Styler.Zipper.html#t:path/0"},{"type":"type","title":"Styler.Zipper.t/0","doc":"","ref":"Styler.Zipper.html#t:t/0"},{"type":"type","title":"Styler.Zipper.tree/0","doc":"","ref":"Styler.Zipper.html#t:tree/0"},{"type":"type","title":"Styler.Zipper.zipper/0","doc":"","ref":"Styler.Zipper.html#t:zipper/0"},{"type":"exception","title":"Styler.StyleError","doc":"Wraps errors raised by Styles during tree traversal.","ref":"Styler.StyleError.html"},{"type":"function","title":"Styler.StyleError.message/1","doc":"","ref":"Styler.StyleError.html#message/1"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\n**Note** Styler's only public API is its usage as a formatter plugin. While you're welcome to play with its internals,\nthey can and will change without that change being reflected in Styler's semantic version.","ref":"changelog.html"},{"type":"extras","title":"main - Changelog","doc":"","ref":"changelog.html#main"},{"type":"extras","title":"1.1.0 - Changelog","doc":"","ref":"changelog.html#1-1-0"},{"type":"extras","title":"Improvements - Changelog","doc":"The big change here is the rewrite/removal of `unless` due to [unless \"eventually\" being deprecated](https://github.com/elixir-lang/elixir/pull/13769#issuecomment-2334878315). Thanks to @janpieper and @ypconstante for bringing this up in #190.\n\n* `unless`: rewrite all `unless` to `if` (#190)\n* `pipes`: optimize `|> Stream.{each|map}(fun) |> Stream.run()` to `|> Enum.each(fun)`","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"* `pipes`: optimizations reducing 2 pipes to 1 no longer squeeze all pipes onto one line (#180)\n* `if`: fix infinite loop rewriting negated if with empty do body `if x != y, do: (), else: :ok` (#196, h/t @itamm15)","ref":"changelog.html#fixes"},{"type":"extras","title":"1.0.0 - Changelog","doc":"Styler's two biggest outstanding bugs have been fixed, both related to compilation breaking during module directive organization. One was references to aliases being moved above where the aliases were declared, and the other was similarly module directives being moved after their uses in module directives.\n\nIn both cases, Styler is now smart enough to auto-apply the fixes we recommended in the old Readme.\n\nOther than that, a slew of powerful new features have been added, the neatest one (in the author's opinion anyways) being Alias Lifting.\n\nThanks to everyone who reported bugs that contributed to all the fixes released in 1.0.0 as well.","ref":"changelog.html#1-0-0"},{"type":"extras","title":"Improvements - Changelog","doc":"#### Alias Lifting\n\nAlong the lines of `Credo.Check.Design.AliasUsage`, Styler now \"lifts\" deeply nested aliases (depth >= 3, ala `A.B.C....`) that are used more than once.\n\nPut plainly, this code:\n\n```elixir\ndefmodule A do\n  def lift_me() do\n    A.B.C.foo()\n    A.B.C.baz()\n  end\nend\n```\n\nwill become\n\n```elixir\ndefmodule A do\n  @moduledoc false\n  alias A.B.C\n\n  def lift_me do\n    C.foo()\n    C.baz()\n  end\nend\n```\n\nTo exclude modules ending in `.Foo` from being lifted, add `styler: [alias_lifting_exclude: [Foo]]` to your `.formatter.exs`\n\n#### Module Attribute Lifting\n\nA long outstanding breakage of a first pass with Styler was breaking directives that relied on module attributes which Styler moved _after_ their uses. Styler now detects these potential breakages and automatically applies our suggested fix, which is creating a variable before the module. This usually happened when folks were using a library that autogenerated their moduledocs for them.\n\nIn code, this module:\n\n```elixir\ndefmodule MyGreatLibrary do\n  @library_options [...]\n  @moduledoc make_pretty_docs(@library_options)\n  use OptionsMagic, my_opts: @library_options\n\n  ...\nend\n```\n\nWill now be styled like so:\n\n```elixir\nlibrary_options = [...]\n\ndefmodule MyGreatLibrary do\n  @moduledoc make_pretty_docs(library_options)\n  use OptionsMagic, my_opts: unquote(library_options)\n\n  @library_options library_options\n\n  ...\nend\n```\n\n#### Mix Config File Organization\n\nStyler now organizes `Mix.Config.config/2,3` stanzas according to erlang term sorting. This helps manage large configuration files, removing the \"where should I put this\" burden from developers AND helping find duplicated configuration stanzas.\n\nSee the moduledoc for `Styler.Style.Configs` for more.\n\n#### Pipe Optimizations\n\n* `Enum.into(x, [])` => `Enum.to_list(x)`\n* `Enum.into(x, [], mapper)` => `Enum.map(x, mapper)`\n* `a |> Enum.map(m) |> Enum.join()` to `map_join(a, m)`. we already did this for `join/2`, but missed the case for `join/1`\n* `lhs |> Enum.reverse() |> Kernel.++(enum)` => `lhs |> Enum.reverse(enum)`\n\n#### `with` styles\n\n* remove `with` structure with no left arrows in its head to be normal code (#174)\n* `with true <- x(), do: y` => `if x(), do: y` (#173)\n\n#### Everything Else\n\n* `if`/`unless`: invert if and unless with `!=` or `!==`, like we do for `!` and `not` #132\n* `@derive`: move `@derive` before `defstruct|schema|embedded_schema` declarations (fixes compiler warning!) #134\n* strings: rewrite double-quoted strings to use `~s` when there's 4+ escaped double-quotes\n  (`\"\\\"\\\"\\\"\\\"\"` -> `~s(\"\"\"\")`) (`Credo.Check.Readability.StringSigils`) #146\n* `Map.drop(foo, [single_key])` => `Map.delete(foo, single_key)` #161 (also in pipes)\n* `Keyword.drop(foo, [single_key])` => `Keyword.delete(foo, single_key)` #161 (also in pipes)","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"* don't blow up on `def function_head_with_no_body_nor_parens` (#185, h/t @ypconstante)\n* fix `with` arrow replacement + redundant body removal creating invalid statements (#184, h/t @JesseHerrick)\n* allow Kernel unary `!` and `not` as valid pipe starts (#183, h/t @nherzing)\n* fix `Map.drop(x, [a | b])` registering as a chance to refactor to `Map.delete`\n* `alias`: expands aliases when moving an alias after another directive that relied on it (#137)\n* module directives: various fixes for unreported obscure crashes\n* pipes: fix a comment-shifting scenario when unpiping\n* `Timex.now/1` will no longer be rewritten to `DateTime.now!/1` due to Timex accepting a wider domain of \"timezones\" than the stdlib (#145, h/t @ivymarkwell)\n* `with`: skip nodes which (unexpectedly) do not contain a `do` body (#158, h/t @DavidB59)\n* `then(&fun/1)`: fix false positives on arithmetic `&1 + x / 1` (#164, h/t @aenglisc)","ref":"changelog.html#fixes"},{"type":"extras","title":"Breaking Changes - Changelog","doc":"* drop support for elixir `1.14`\n* ModuleDirectives: group callback attributes (`before_compile after_compile after_verify`) with nondirectives (previously, were grouped with `use`, their relative order maintained). to keep the desired behaviour, you can make new `use` macros that wrap these callbacks. Apologies if this makes using Styler untenable for your codebase, but it's probably not a good tool for macro-heavy libraries.\n* sorting configs for the first time can change your configuration; see [Mix Configs docs](docs/mix_configs.md) for more","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Styler","doc":"[![Hex.pm](https://img.shields.io/hexpm/v/styler)](https://hex.pm/packages/styler)\n[![Hexdocs.pm](https://img.shields.io/badge/docs-hexdocs.pm-purple)](https://hexdocs.pm/styler)\n[![Github.com](https://github.com/adobe/elixir-styler/actions/workflows/ci.yml/badge.svg)](https://github.com/adobe/elixir-styler/actions)\n\n# Styler\n\nStyler is an Elixir formatter plugin that's combination of `mix format` and `mix credo`, except instead of telling\nyou what's wrong, it just rewrites the code for you to fit its style rules.\n\nYou can learn more about the history, purpose and implementation of Styler from our talk: [Styler: Elixir Style-Guide Enforcer @ GigCity Elixir 2023](https://www.youtube.com/watch?v=6pF8Hl5EuD4)","ref":"readme.html"},{"type":"extras","title":"Features - Styler","doc":"- auto-fixes [many credo rules](docs/credo.md), meaning you can turn them off to speed credo up\n- [keeps a strict module layout](docs/module_directives.md#directive-organization)\n  - alphabetizes module directives\n- [extracts repeated aliases](docs/module_directives.md#alias-lifting)\n- [makes your pipe chains pretty as can be](docs/pipes.md)\n  - pipes and unpipes function calls based on the number of calls\n  - optimizes standard library calls (`a |> Enum.map(m) |> Enum.into(Map.new)` => `Map.new(a, m)`)\n- replaces strings with sigils when the string has many escaped quotes\n- ... and so much more\n\n[See our Rewrites documentation on hexdocs](https://hexdocs.pm/styler/styles.html)","ref":"readme.html#features"},{"type":"extras","title":"Who is Styler for? - Styler","doc":"Styler was designed for a **large team (40+ engineers) working in a single codebase. It helps remove fiddly code review comments and removes failed linter CI slowdowns, helping teams get things done faster. Teams in similar situations might appreciate Styler.\n\nIts automations are also extremely valuable for taming legacy elixir codebases or just refactoring in general. Some of its rewrites have inspired code actions in elixir language servers.\n\nConversely, Styler probably _isn't_ a good match for:\n\n- experimental, macro-heavy codebases\n- teams that don't care about code standards","ref":"readme.html#who-is-styler-for"},{"type":"extras","title":"Installation - Styler","doc":"Add `:styler` as a dependency to your project's `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:styler, \"~> 1.1\", only: [:dev, :test], runtime: false},\n  ]\nend\n```\n\nThen add `Styler` as a plugin to your `.formatter.exs` file\n\n```elixir\n[\n  plugins: [Styler]\n]\n```\n\nAnd that's it! Now when you run `mix format` you'll also get the benefits of Styler's Stylish Stylings.\n\n**Speed**: Expect the first run to take some time as `Styler` rewrites violations of styles and bottlenecks on disk I/O. Subsequent formats formats won't take noticeably more time.","ref":"readme.html#installation"},{"type":"extras","title":"Configuration - Styler","doc":"Styler can be configured in your `.formatter.exs` file\n\n```elixir\n[\n  plugins: [Styler],\n  styler: [\n    alias_lifting_exclude: [...]\n  ]\n]\n```\n\nStyler's only current configuration option is `:alias_lifting_exclude`, which accepts a list of atoms to _not_ lift. See the [Module Directive documentation](docs/module_directives.md#alias-lifting) for more.\n\n#### No Credo-Style Enable/Disable\n\nStyler [will not add configuration](https://github.com/adobe/elixir-styler/pull/127#issuecomment-1912242143) for ad-hoc enabling/disabling of rewrites. Sorry! Its implementation simply does not support that approach. There are however many forks out there that have attempted this; please explore the [Github forks tab](https://github.com/adobe/elixir-styler/forks) to see if there's a project that suits your needs or that you can draw inspiration from.\n\nUltimately Styler is @adobe's internal tool that we're happy to share with the world. We're delighted if you like it as is, and just as excited if it's a starting point for you to make something even better for yourself.","ref":"readme.html#configuration"},{"type":"extras","title":"WARNING: Styler can change the behaviour of your program! - Styler","doc":"In some cases, this can introduce bugs. It goes without saying, but look over your changes before committing to main :)\n\nA simple example of a way Styler changes the behaviour of code is the following rewrite:\n\n```elixir\n# Before: this case statement...\ncase foo do\n  true -> :ok\n  false -> :error\nend\n\n# After: ... is rewritten by Styler to be an if statement!.\nif foo do\n  :ok\nelse\n  :error\nend\n```\n\nThese programs are not semantically equivalent. The former would raise if `foo` returned any value other than `true` or `false`, while the latter blissfully completes.\n\nHowever, Styler is about _style_, and the `if` statement is (in our opinion) of much better style. If the exception behaviour was intentional on the code author's part, they should have written the program like this:\n\n```elixir\ncase foo do\n  true -> :ok\n  false -> :error\n  other -> raise \"expected `true` or `false`, got: #{inspect other}\"\nend\n```\n\nAlso good style! But Styler assumes that most of the time people just meant the `if` equivalent of the code, and so makes that change. If issues like this bother you, Styler probably isn't the tool you're looking for.\n\nOther ways Styler can change your program:\n\n- [`with` statement rewrites](https://github.com/adobe/elixir-styler/issues/186)\n- [config file sorting](https://hexdocs.pm/styler/mix_configs.html#this-can-break-your-program)\n- and likely other ways. stay safe out there!","ref":"readme.html#warning-styler-can-change-the-behaviour-of-your-program"},{"type":"extras","title":"Thanks & Inspiration - Styler","doc":"### [Sourceror](https://github.com/doorgan/sourceror/)\n\nStyler's first incarnation was as one-off scripts to rewrite an internal codebase to allow Credo rules to be turned on.\n\nThese rewrites were entirely powered by the terrific `Sourceror` library.\n\nWhile `Styler` no longer relies on `Sourceror`, we're grateful for its author's help with those scripts, the inspiration\nSourceror provided in showing us what was possible, and the changes to the Elixir AST APIs that it drove.\n\nStyler's [AST-Zipper](`m:Styler.Zipper`) implementation in this project was forked from Sourceror. Zipper has been a crucial\npart of our ability to ergonomically zip around (heh) Elixir AST.\n\n### [Credo](https://github.com/rrrene/credo/)\n\nWe never would've bothered trying to rewrite our codebase if we didn't have Credo rules we wanted to apply.\n\nCredo's tests and implementations were referenced for implementing Styles that took the work the rest of the way.\n\nThanks to Credo & the Elixir community at large for coalescing around many of these Elixir style credos.","ref":"readme.html#thanks-inspiration"},{"type":"extras","title":"Basic Styles","doc":"# Simple (Single Node) Styles\n\nFunction Performance & Readability Optimizations\n\nOptimizing for either performance or readability, probably both!\nThese apply to the piped versions as well","ref":"styles.html"},{"type":"extras","title":"Strings to Sigils - Basic Styles","doc":"Rewrites strings with 4 or more escaped quotes to string sigils with an alternative delimiter.\nThe delimiter will be one of `\" ( { | [ ' < /`, chosen by which would require the fewest escapes, and otherwise preferred in the order listed.\n\n```elixir\n# Before\n\"{\\\"errors\\\":[\\\"Not Authorized\\\"]}\"\n# Styled\n~s({\"errors\":[\"Not Authorized\"]})\n```","ref":"styles.html#strings-to-sigils"},{"type":"extras","title":"Large Base 10 Numbers - Basic Styles","doc":"Style base 10 numbers with 5 or more digits to have a `_` every three digits.\nFormatter already does this except it doesn't rewrite \"typos\" like `100_000_0`.\n\nIf you're concerned that this breaks your team's formatting for things like \"cents\" (like \"$100\" being written as `100_00`),\nconsider using a library made for denoting currencies rather than raw elixir integers.\n\n| Before | After |\n|--------|-------|\n| `10000 ` | `10_000`|\n| `1_0_0_0_0` | `10_000` (elixir's formatter leaves the former as-is)|\n| `-543213 ` | `-543_213`|\n| `123456789 ` | `123_456_789`|\n| `55333.22 ` | `55_333.22`|\n| `-123456728.0001 ` | `-123_456_728.0001`|\n\n## `Enum.into` -> `X.new`\n\nThis rewrite is applied when the collectable is a new map, keyword list, or mapset via `Enum.into/2,3`.\n\nThis is an improvement for the reader, who gets a more natural language expression: \"make a new map from enum\" vs \"enumerate enum and collect its elements into a new map\"\n\nNote that all of the examples below also apply to pipes (`enum |> Enum.into(...)`)\n\n| Before | After |\n|--------|-------|\n| `Enum.into(enum, %{})` | `Map.new(enum)`|\n| `Enum.into(enum, Map.new())` | `Map.new(enum)`|\n| `Enum.into(enum, Keyword.new())` | `Keyword.new(enum)`|\n| `Enum.into(enum, MapSet.new())` | `Keyword.new(enum)`|\n| `Enum.into(enum, %{}, fn x -> {x, x} end)` | `Map.new(enum, fn x -> {x, x} end)`|\n| `Enum.into(enum, [])` | `Enum.to_list(enum)` |\n| `Enum.into(enum, [], mapper)` | `Enum.map(enum, mapper)`|","ref":"styles.html#large-base-10-numbers"},{"type":"extras","title":"Map/Keyword.merge w/ single key literal -> X.put - Basic Styles","doc":"`Keyword.merge` and `Map.merge` called with a literal map or keyword argument with a single key are rewritten to the equivalent `put`, a cognitively simpler function.\n\n```elixir\n# Before\nKeyword.merge(kw, [key: :value])\n# Styled\nKeyword.put(kw, :key, :value)\n\n# Before\nMap.merge(map, %{key: :value})\n# Styled\nMap.put(map, :key, :value)\n\n# Before\nMap.merge(map, %{key => value})\n# Styled\nMap.put(map, key, value)\n\n# Before\nmap |> Map.merge(%{key: value}) |> foo()\n# Styled\nmap |> Map.put(:key, value) |> foo()\n```","ref":"styles.html#map-keyword-merge-w-single-key-literal-x-put"},{"type":"extras","title":"Map/Keyword.drop w/ single key -> X.delete - Basic Styles","doc":"In the same vein as the `merge` style above, `[Map|Keyword].drop/2` with a single key to drop are rewritten to use `delete/2`\n```elixir\n# Before\nMap.drop(map, [key])\n# Styled\nMap.delete(map, key)\n\n# Before\nKeyword.drop(kw, [key])\n# Styled\nKeyword.delete(kw, key)\n```\n\n## `Enum.reverse/1` and concatenation -> `Enum.reverse/2`\n\n`Enum.reverse/2` optimizes a two-step reverse and concatenation into a single step.\n\n```elixir\n# Before\nEnum.reverse(foo) ++ bar\n# Styled\nEnum.reverse(foo, bar)\n\n# Before\nbaz |> Enum.reverse() |> Enum.concat(bop)\n# Styled\nEnum.reverse(baz, bop)\n```\n\n## `Timex.now/0` ->` DateTime.utc_now/0`\n\nTimex certainly has its uses, but knowing what stdlib date/time struct is returned by `now/0` is a bit difficult!\n\nWe prefer calling the actual function rather than its rename in Timex, helping the reader by being more explicit.\n\nThis also hews to our internal styleguide's \"Don't make one-line helper functions\" guidance.\n\n## `DateModule.compare/2` -> `DateModule.[before?|after?]`\n\nAgain, the goal is readability and maintainability. `before?/2` and `after?/2` were implemented long after `compare/2`,\nso it's not unusual that a codebase needs a lot of refactoring to be brought up to date with these new functions.\nThat's where Styler comes in!\n\nThe examples below use `DateTime.compare/2`, but the same is also done for `NaiveDateTime|Time|Date.compare/2`\n\n```elixir\n# Before\nDateTime.compare(start, end_date) == :gt\n# Styled\nDateTime.after?(start, end_date)\n\n# Before\nDateTime.compare(start, end_date) == :lt\n# Styled\nDateTime.before?(start, end_date)\n```","ref":"styles.html#map-keyword-drop-w-single-key-x-delete"},{"type":"extras","title":"Implicit Try - Basic Styles","doc":"Styler will rewrite functions whose entire body is a try/do to instead use the implicit try syntax, per Credo's `Credo.Check.Readability.PreferImplicitTry`\n\nThe following example illustrates the most complex case, but Styler happily handles just basic try do/rescue bodies just as easily.","ref":"styles.html#implicit-try"},{"type":"extras","title":"Before - Basic Styles","doc":"```elixir\ndef foo() do\n  try do\n    uh_oh()\n  rescue\n    exception -> {:error, exception}\n  catch\n    :a_throw -> {:error, :threw!}\n  else\n    try_has_an_else_clause? -> {:did_you_know, try_has_an_else_clause?}\n  after\n    :done\n  end\nend\n```","ref":"styles.html#before"},{"type":"extras","title":"After - Basic Styles","doc":"```elixir\ndef foo() do\n  uh_oh()\nrescue\n  exception -> {:error, exception}\ncatch\n  :a_throw -> {:error, :threw!}\nelse\n  try_has_an_else_clause? -> {:did_you_know, try_has_an_else_clause?}\nafter\n  :done\nend\n```","ref":"styles.html#after"},{"type":"extras","title":"Remove parenthesis from 0-arity function & macro definitions - Basic Styles","doc":"The author of the library disagrees with this style convention :) BUT, the wonderful thing about Styler is it lets you write code how _you_ want to, while normalizing it for reading for your entire team. The most important thing is not having to think about the style, and instead focus on what you're trying to achieve.\n\n```elixir\n# Before\ndef foo()\ndefp foo()\ndefmacro foo()\ndefmacrop foo()\n\n# Styled\ndef foo\ndefp foo\ndefmacro foo\ndefmacrop foo\n```","ref":"styles.html#remove-parenthesis-from-0-arity-function-macro-definitions"},{"type":"extras","title":"Elixir Deprecation Rewrites - Basic Styles","doc":"","ref":"styles.html#elixir-deprecation-rewrites"},{"type":"extras","title":"1.15+ - Basic Styles","doc":"| Before | After |\n|--------|-------|\n| `Logger.warn` | `Logger.warning`|\n| `Path.safe_relative_to/2` | `Path.safe_relative/2`|\n| `~R/my_regex/` | `~r/my_regex/`|\n| `Enum/String.slice/2` with decreasing ranges | add explicit steps to the range * |\n| `Date.range/2` with decreasing range | `Date.range/3` *|\n| `IO.read/bin_read` with `:all` option | replace `:all` with `:eof`|\n\n\\* For both of the \"decreasing range\" changes, the rewrite can only be applied if the range is being passed as an argument to the function.","ref":"styles.html#1-15"},{"type":"extras","title":"1.16+ - Basic Styles","doc":"File.stream! `:line` and `:bytes` deprecation\n\n```elixir\n# Before\nFile.stream!(path, [encoding: :utf8, trim_bom: true], :line)\n# Styled\nFile.stream!(path, :line, encoding: :utf8, trim_bom: true)\n```","ref":"styles.html#1-16"},{"type":"extras","title":"Putting variable matching on the right - Basic Styles","doc":"```elixir\n# Before\ncase foo do\n  bar = %{baz: baz? = true} -> :baz?\n  opts = [[a = %{}] | _] -> a\nend\n# Styled:\ncase foo do\n  %{baz: true = baz?} = bar -> :baz?\n  [[%{} = a] | _] = opts -> a\nend\n\n# Before\nwith {:ok, result = %{}} <- foo, do: result\n# Styled\nwith {:ok, %{} = result} <- foo, do: result\n\n# Before\ndef foo(bar = %{baz: baz? = true}, opts = [[a = %{}] | _]), do: :ok\n# Styled\ndef foo(%{baz: true = baz?} = bar, [[%{} = a] | _] = opts), do: :ok\n```","ref":"styles.html#putting-variable-matching-on-the-right"},{"type":"extras","title":"Drops superfluous `= _` in pattern matching - Basic Styles","doc":"```elixir\n# Before\ndef foo(_ = bar), do: bar\n# Styled\ndef foo(bar), do: bar\n\n# Before\ncase foo do\n  _ = bar -> :ok\nend\n# Styled\ncase foo do\n  bar -> :ok\nend\n```","ref":"styles.html#drops-superfluous-_-in-pattern-matching"},{"type":"extras","title":"Use Implicit Try - Basic Styles","doc":"```elixir\n# before\ndef foo d\n  try do\n    throw_ball()\n  catch\n    :ball -> :caught\n  end\nend\n\n# Styled:\ndef foo d\n  throw_ball()\ncatch\n  :ball -> :caught\nend\n```","ref":"styles.html#use-implicit-try"},{"type":"extras","title":"Shrink Function Definitions to One Line When Possible - Basic Styles","doc":"```elixir\n# Before\n\ndef save(\n       # Socket comment\n       %Socket{assigns: %{user: user, live_action: :new}} = initial_socket,\n       # Params comment\n       params\n     ),\n     do: :ok\n\n# Styled\n\n# Socket comment\n# Params comment\ndef save(%Socket{assigns: %{user: user, live_action: :new}} = initial_socket, params), do: :ok\n```","ref":"styles.html#shrink-function-definitions-to-one-line-when-possible"},{"type":"extras","title":"Pipe Chains","doc":"# Pipe Chains","ref":"pipes.html"},{"type":"extras","title":"Pipe Start - Pipe Chains","doc":"Styler will ensure that the start of a pipechain is a 0-arity function, a raw value, or a variable.\n\n```elixir\nEnum.at(enum, 5)\n|> IO.inspect()\n\n# Styled:\nenum\n|> Enum.at(5)\n|> IO.inspect()\n```\n\nIf the start of a pipe is a block expression, styler will create a new variable to store the result of that expression and make that variable the start of the pipe.\n\n```elixir\nif a do\n  b\nelse\n  c\nend\n|> Enum.at(4)\n|> IO.inspect()\n\n# Styled:\nif_result =\n  if a do\n    b\n  else\n    c\n  end\n\nif_result\n|> Enum.at(4)\n|> IO.inspect()\n```","ref":"pipes.html#pipe-start"},{"type":"extras","title":"Add parenthesis to function calls in pipes - Pipe Chains","doc":"```elixir\na |> b |> c |> d\n# Styled:\na |> b() |> c() |> d()\n```","ref":"pipes.html#add-parenthesis-to-function-calls-in-pipes"},{"type":"extras","title":"Remove Unnecessary `then/2` - Pipe Chains","doc":"When the piped argument is being passed as the first argument to the inner function, there's no need for `then/2`.\n\n```elixir\na |> then(&f(&1, ...)) |> b()\n# Styled:\na |> f(...) |> b()\n```\n\n- add parens to function calls `|> fun |>` => `|> fun() |>`","ref":"pipes.html#remove-unnecessary-then-2"},{"type":"extras","title":"Add `then/2` when defining and calling anonymous functions in pipes - Pipe Chains","doc":"```elixir\na |> (fn x -> x end).() |> c()\n# Styled:\na |> then(fn x -> x end) |> c()\n```","ref":"pipes.html#add-then-2-when-defining-and-calling-anonymous-functions-in-pipes"},{"type":"extras","title":"Piped function optimizations - Pipe Chains","doc":"Two function calls into one! Fewer steps is always nice.\n\n```elixir\n# reverse |> concat => reverse/2\na |> Enum.reverse() |> Enum.concat(enum) |> ...\n# Styled:\na |> Enum.reverse(enum) |> ...\n\n# filter |> count => count(filter)\na |> Enum.filter(filterer) |> Enum.count() |> ...\n# Styled:\na |> Enum.count(filterer) |> ...\n\n# map |> join => map_join\na |> Enum.map(mapper) |> Enum.join(joiner) |> ...\n# Styled:\na |> Enum.map_join(joiner, mapper) |> ...\n\n# Enum.map |> X.new() => X.new(mapper)\n# where X is one of: Map, MapSet, Keyword\na |> Enum.map(mapper) |> Map.new() |> ...\n# Styled:\na |> Map.new(mapper) |> ...\n\n# Enum.map |> Enum.into(empty_collectable) => X.new(mapper)\n# Where empty_collectable is one of `%{}`, `Map.new()`, `Keyword.new()`, `MapSet.new()`\n# Given:\na |> Enum.map(mapper) |> Enum.into(%{}) |> ...\n# Styled:\na |> Map.new(mapper) |> ...\n\n# Given:\na |> b() |> Stream.each(fun) |> Stream.run()\na |> b() |> Stream.map(fun) |> Stream.run()\n# Styled:\na |> b() |> Enum.each(fun)\na |> b() |> Enum.each(fun)\n```","ref":"pipes.html#piped-function-optimizations"},{"type":"extras","title":"Unpiping Single Pipes - Pipe Chains","doc":"Styler rewrites pipechains with a single pipe to be function calls. Notably, this rule combined with the optimizations rewrites above means some chains with more than one pipe will also become function calls.\n\n```elixir\nfoo = bar |> baz()\n# Styled:\nfoo = baz(bar)\n\nmap = a |> Enum.map(mapper) |> Map.new()\n# Styled:\nmap = Map.new(a, mapper)\n```","ref":"pipes.html#unpiping-single-pipes"},{"type":"extras","title":"Control Flow Macros (if, case, ...)","doc":"# Control Flow Macros (`case`, `if`, `unless`, `cond`, `with`)\n\nElixir's Kernel documentation refers to these structures as \"macros for control-flow\".\nWe often refer to them as \"blocks\" in our changelog, which is a much worse name, to be sure.\n\nYou're likely here just to see what Styler does, in which case, please [click here to skip](#if-and-unless) the following manifesto on our philosophy regarding the usage of these macros.","ref":"control_flow_macros.html"},{"type":"extras","title":"Which Control Flow Macro Should I Use? - Control Flow Macros (if, case, ...)","doc":"The number of \"blocks\" in Elixir means there are many ways to write semantically equivalent code, often leaving developers [in the dark as to which structure they should use.](https://www.reddit.com/r/elixir/comments/1ctbtcl/i_am_completely_lost_when_it_comes_to_which/)\n\nWe believe readability is enhanced by using the simplest api possible, whether we're talking about internal module function calls or standard-library macros.","ref":"control_flow_macros.html#which-control-flow-macro-should-i-use"},{"type":"extras","title":"use `case`, `if`, or `cond` when... - Control Flow Macros (if, case, ...)","doc":"We advocate for `case` and `if` as the first tools to be considered for any control flow as they are the two simplest blocks. If a branch _can_ be expressed with an `if` statement, it _should_ be. Otherwise, `case` is the next best choice. In situations where developers might reach for an `if/elseif/else` block in other languages, `cond do` should be used.\n\n(`cond do` seems to see a paucity of use in the language, but many complex nested expressions or with statements can be improved by replacing them with a `cond do`).","ref":"control_flow_macros.html#use-case-if-or-cond-when"},{"type":"extras","title":"use `unless` when... - Control Flow Macros (if, case, ...)","doc":"Never! `unless` [is being deprecated](https://github.com/elixir-lang/elixir/pull/13769#issuecomment-2334878315) and so should not be used.","ref":"control_flow_macros.html#use-unless-when"},{"type":"extras","title":"use `with` when... - Control Flow Macros (if, case, ...)","doc":"> `with` great power comes great responsibility\n>\n> - Uncle Ben\n\nAs the most powerful of the Kernel control-flow expressions, `with` requires the most cognitive overhead to understand. Its power means that we can use it as a replacement for anything we might express using a `case`, `if`, or `cond` (especially with the liberal application of small private helper functions).\n\nUnfortunately, this has lead to a proliferation of `with` in codebases where simpler expressions would have sufficed, meaning a lot of Elixir code ends up being harder for readers to understand than it needs to be.\n\nThus, `with` is the control-flow structure of last resort. We advocate that `with` **should only be used when more basic expressions do not suffice or become overly verbose**. As for verbosity, we subscribe to the [Chris Keathley school of thought](https://www.youtube.com/watch?v=l-8ghbdRB1w) that judicious nesting of control flow blocks within a function isn't evil and more-often-than-not is superior to spreading implementation over many small single-use functions. We'd even go so far as to suggest that cyclomatic complexity is an inexact measure of code quality, with more than a few false negatives and many false positives.\n\n`with` is a great way to unnest multiple `case` statements when every failure branch of those statements results in the same error. This is easily and succinctly expressed with `with`'s `else` block: `else (_ -> :error)`. As Keathley says though, [Avoid Else In With Blocks](https://keathley.io/blog/good-and-bad-elixir.html#avoid-else-in-with-blocks). Having multiple else clauses \"means that the error conditions matter. Which means that you don’t want `with` at all. You want `case`.\"\n\nIt's acceptable to use one-line `with` statements (eg `with {:ok, _} <- Repo.update(changeset), do: :ok`) to signify that other branches are uninteresting or unmodified by your code, but ultimately that can hide the possible returns of a function from the reader, making it more onerous to debug all possible branches of the code in their mental model of the function. In other words, ideally all function calls in a `with` statement head have obvious error types for the reader, leaving their omission in the code acceptable as the reader feels no need to investigate further. The example at the start of this paragraph with an `Ecto.Repo` call is a good example, as most developers in a codebase using Ecto are expected to be familiar with its basic API.\n\nUsing `case` rather than `with` for branches with unusual failure types can help document code as well as save the reader time in tracking down types. For example, replacing the following with a `with` statement that only matched against the `{:ok, _}` tuple would hide from readers that an atypically-shaped 3-tuple is returned when things go wrong.\n\n```elixir\ncase some_http_call() do\n  {:ok, _response} -> :ok\n  {:error, http_error, response} -> {:error, http_error, response}\nend\n```\n\n## `if` and `unless`\n\nStyler removes `else: nil` clauses:\n\n```elixir\nif a, do: b, else: nil\n# styled:\nif a, do: b\n```","ref":"control_flow_macros.html#use-with-when"},{"type":"extras","title":"Negation Inversion - Control Flow Macros (if, case, ...)","doc":"Styler removes negators in the head of `if` and `unless` statements by \"inverting\" the statement.\nThe following operators are considered \"negators\": `!`, `not`, `!=`, `!==`\n\n\nExamples:\n\n```elixir\n# negated `if` statement with no `else` clause are rewritten to `unless`\nif not x, do: y\n# Styled:\nunless x, do: y\n\n# negated `if` statements with an `else` clause have their clauses inverted and negation removed\nif !x, do: y, else: z\n# Styled:\nif x, do: z, else: y\n\n# negated `unless` statements are rewritten to `if`\nunless x != y, do: z\n# B styled:\nif x == y, do: z\n\n# `unless` with `else` is verboten; these are always rewritten to `if` statements\nunless x, do: y, else: z\n# styled:\nif x, do: z, else: y\n```\n\nBecause elixir relies on truthy/falsey values for its `if` statements, boolean casting is unnecessary and so double negation is simply removed.\n\n```elixir\nif !!x, do: y\n# styled:\nif x, do: y\n```\n\n## `case`\n\n### \"Erlang heritage\" `case` true/false -> `if`\n\nTrivial true/false `case` statements are rewritten to `if` statements. While this results in a [semantically different program](https://github.com/rrrene/credo/issues/564#issue-338349517), we argue that it results in a better program for maintainability. If the developer wants their case statement to raise when receiving a non-boolean value as a feature of the program, they would better serve their callers by raising something more descriptive.\n\nIn other words, Styler leaves the code with better style, trumping obscure exception design :)\n\n```elixir\n# Styler will rewrite this even if the clause order is flipped,\n# and if the `false` is replaced with a wildcard (`_`)\ncase foo do\n  true -> :ok\n  false -> :error\nend\n\n# styled:\nif foo do\n  :ok\nelse\n  :error\nend\n```\n\nPer the argument above, if the `if` statement is an incorrect rewrite for your program, we recommend this manual fix rewrite:\n\n```elixir\ncase foo do\n  true -> :ok\n  false -> :error\n  other -> raise \"expected `true` or `false`, got: #{inspect other}\"\nend\n```\n\n## `cond`\n\nStyler has only one `cond` statement rewrite: replace 2-clause statements with `if` statements.\n\n```elixir\n# Given\ncond do\n  a -> b\n  true -> c\nend\n# Styled\nif a do\n  b\nelse\n  c\nend\n```\n\n## `with`\n\n`with` statements are extremely expressive. Styler tries to remove any unnecessary complexity from them in the following ways.","ref":"control_flow_macros.html#negation-inversion"},{"type":"extras","title":"Remove Identity Else Clause - Control Flow Macros (if, case, ...)","doc":"Like if statements with `nil` as their else clause, the identity `else` clause is the default for `with` statements and so is removed.\n\n```elixir\n# Given\nwith :ok <- b(), :ok <- b() do\n  foo()\nelse\n  error -> error\nend\n# Styled:\nwith :ok <- b(), :ok <- b() do\n  foo()\nend\n```","ref":"control_flow_macros.html#remove-identity-else-clause"},{"type":"extras","title":"Remove The Statement Entirely - Control Flow Macros (if, case, ...)","doc":"While you might think \"surely this kind of code never appears in the wild\", it absolutely does. Typically it's the result of someone refactoring a pattern away and not looking at the larger picture and realizing that the with statement now serves no purpose.\n\nMaybe someday the compiler will warn about these use cases. Until then, Styler to the rescue.\n\n```elixir\n# Given:\nwith a <- b(),\n     c <- d(),\n     e <- f(),\n     do: g,\n     else: (_ -> h)\n# Styled:\na = b()\nc = d()\ne = f()\ng\n\n# Given\nwith value <- arg do\n  value\nend\n# Styled:\narg\n```","ref":"control_flow_macros.html#remove-the-statement-entirely"},{"type":"extras","title":"Replace `_ <- rhs` with `rhs` - Control Flow Macros (if, case, ...)","doc":"This is another case of \"less is more\" for the reader.\n\n```elixir\n# Given\nwith :ok <- x,\n     _ <- y(),\n     {:ok, _} <- z do\n  :ok\nend\n# Styled:\nwith :ok <- x,\n     y(),\n     {:ok, _} <- z do\n  :ok\nend\n```","ref":"control_flow_macros.html#replace-_-rhs-with-rhs"},{"type":"extras","title":"Replace non-branching `bar <-` with `bar =` - Control Flow Macros (if, case, ...)","doc":"`<-` is for branching. If the lefthand side is the trivial match (a bare variable), Styler rewrites it to use the `=` operator instead.\n\n```elixir\n# Given\nwith :ok <- foo(),\n     bar <- baz(),\n     :ok <- woo(),\n     do: {:ok, bar}\n# Styled\n with :ok <- foo(),\n      bar = baz(),\n      :ok <- woo(),\n      do: {:ok, bar}\n```","ref":"control_flow_macros.html#replace-non-branching-bar-with-bar"},{"type":"extras","title":"Move assignments from `with` statement head - Control Flow Macros (if, case, ...)","doc":"Just because any program _could_ be written entirely within the head of a `with` statement doesn't mean it should be!\n\nStyler moves assignments that aren't trapped between `<-` outside of the head. Combined with the non-pattern-matching replacement above, we get the following:\n\n```elixir\n# Given\nwith foo <- bar,\n     x = y,\n     :ok <- baz,\n     bop <- boop,\n     :ok <- blop,\n     foo <- bar,\n     :success = hope_this_works! do\n  :ok\nend\n# Styled:\nfoo = bar\nx = y\n\nwith :ok <- baz,\n     bop = boop,\n     :ok <- blop do\n  foo = bar\n  :success = hope_this_works!\n  :ok\nend\n```","ref":"control_flow_macros.html#move-assignments-from-with-statement-head"},{"type":"extras","title":"Remove redundant final clause - Control Flow Macros (if, case, ...)","doc":"If the pattern of the final clause of the head is also the `with` statements `do` body, styler nixes the final match and makes the right hand side of the clause into the do body.\n\n```elixir\n# Given\nwith {:ok, a} <- foo(),\n     {:ok, b} <- bar(a) do\n  {:ok, b}\nend\n# Styled:\nwith {:ok, a} <- foo() do\n  bar(a)\nend\n```","ref":"control_flow_macros.html#remove-redundant-final-clause"},{"type":"extras","title":"Replace with `case` - Control Flow Macros (if, case, ...)","doc":"A `with` statement with a single clause in the head and an `else` body is really just a `case` statement putting on airs.\n\n```elixir\n# Given:\nwith :ok <- foo do\n  :success\nelse\n  :fail -> :failure\n  error -> error\nend\n# Styled:\ncase foo do\n  :ok -> :success\n  :fail -> :failure\n  error -> error\nend\n```","ref":"control_flow_macros.html#replace-with-case"},{"type":"extras","title":"Replace with `if` - Control Flow Macros (if, case, ...)","doc":"Given Styler rewrites trivial `case` to `if`, it shouldn't be a surprise that that same rule means that `with` can be rewritten to `if` in some cases.\n\n```elixir\n# Given:\nwith true <- foo(), bar <- baz() do\n  {:ok, bar}\nelse\n  _ -> :error\nend\n# Styled:\nif foo() do\n  bar = baz()\n  {:ok, bar}\nelse\n  :error\nend\n```","ref":"control_flow_macros.html#replace-with-if"},{"type":"extras","title":"Mix Configs (config/config.exs, ...)","doc":"# Mix Configs\n\nMix Config files have their config stanzas sorted. Similar to the sorting of aliases, this delivers consistency to an otherwise arbitrary world, and can even help catch bugs like configuring the same key multiple times.\n\nA file is considered a config file if\n\n1. its path matches `~r|config/.*\\.exs|` `~r|rel/overlays/.*\\.exs|`\n2. the file has `import Config`\n\nOnce a file is detected as a mix config, its `config/2,3` stanzas are grouped and ordered like so:\n\n- group config stanzas separated by assignments (`x = y`) together\n- sort each group according to erlang term sorting\n- move all existing assignments between the config stanzas to above the stanzas (without changing their ordering)","ref":"mix_configs.html"},{"type":"extras","title":"THIS CAN BREAK YOUR PROGRAM - Mix Configs (config/config.exs, ...)","doc":"It's important to double check your configuration after running Styler on it for the first time.\n\n**First Use Advice**: To limit the size of changes Styler submits to a codebase, we recommend formatting only a few (or a single) files at a time and making pull requests for each. Only commit Styler as a new formatter plugin once each of these more dangerous changes has been safely committed to the codebase.\n\nImagine your application configures the same value twice, once with an invalid or application breaking value, and then again with a correct value, like so:\n\n```elixir\nstring = \"i am a string\"\natom = :i_am_an_atom\n\nconfig :my_app, value_must_be_an_atom: string\n...\n...\nconfig :my_app, value_must_be_an_atom: atom\n```\n\nWhen styler sorts the configuration file, this dormant mistake can become a bug if the sorting changes the order such that the invalid value takes precedence (aka comes last)\n\n```elixir\nstring = \"i am a string\"\natom = :i_am_an_atom\n\n# The value that must be an atom is now a string!\nconfig :my_app, value_must_be_an_atom: atom\nconfig :my_app, value_must_be_an_atom: string\n```","ref":"mix_configs.html#this-can-break-your-program"},{"type":"extras","title":"Examples - Mix Configs (config/config.exs, ...)","doc":"Sorts configs by erlang term ordering:\n\n```elixir\n# Given\nimport Config\n\nconfig :z, :x, :c\nconfig :a, :b, :c\nconfig :y, :x, :z\nconfig :a, :c, :d\n\n# Styled:\nimport Config\n\nconfig :a, :b, :c\nconfig :a, :c, :d\n\nconfig :y, :x, :z\n\nconfig :z, :x, :c\n```\n\nNon-config statements break the file up into chunks, where each chunk is sorted separately relative to itself.\n\n```elixir\n# Given\nimport Config\n\nconfig :z, :x, :c\nconfig :a, :b, :c\nvar = \"value\"\nconfig :y, :x, var\nconfig :a, :c, var\n\n# Styled:\nimport Config\n\nconfig :a, :b, :c\nconfig :z, :x, :c\n\nvar = \"value\"\n\nconfig :a, :c, var\nconfig :y, :x, var\n```","ref":"mix_configs.html#examples"},{"type":"extras","title":"Module Directives (use, alias, ...)","doc":"","ref":"module_directives.html"},{"type":"extras","title":"Adds Moduledoc - Module Directives (use, alias, ...)","doc":"Adds `@moduledoc false` to modules without a moduledoc unless the module's name ends with one of the following:\n\n* `Test`\n* `Mixfile`\n* `MixProject`\n* `Controller`\n* `Endpoint`\n* `Repo`\n* `Router`\n* `Socket`\n* `View`\n* `HTML`\n* `JSON`","ref":"module_directives.html#adds-moduledoc"},{"type":"extras","title":"Directive Expansion - Module Directives (use, alias, ...)","doc":"Expands `Module.{SubmoduleA, SubmoduleB}` to their explicit forms for ease of searching.\n\n```elixir\n# Before\nimport Foo.{Bar, Baz, Bop}\nalias Foo.{Bar, Baz.A, Bop}\n\n# After\nimport Foo.Bar\nimport Foo.Baz\nimport Foo.Bop\n\nalias Foo.Bar\nalias Foo.Baz.A\nalias Foo.Bop\n```","ref":"module_directives.html#directive-expansion"},{"type":"extras","title":"Directive Organization - Module Directives (use, alias, ...)","doc":"Modules directives are sorted into the following order:\n\n* `@shortdoc`\n* `@moduledoc` (adds `@moduledoc false`)\n* `@behaviour`\n* `use`\n* `import` (sorted alphabetically)\n* `alias` (sorted alphabetically)\n* `require` (sorted alphabetically)\n* everything else (order unchanged)","ref":"module_directives.html#directive-organization"},{"type":"extras","title":"Before - Module Directives (use, alias, ...)","doc":"```elixir\ndefmodule Foo do\n  @behaviour Lawful\n  alias A.A\n  require A\n\n  use B\n\n  def c(x), do: y\n\n  import C\n  @behaviour Chaotic\n  @doc \"d doc\"\n  def d do\n    alias X.X\n    alias H.H\n\n    alias Z.Z\n    import Ecto.Query\n    X.foo()\n  end\n  @shortdoc \"it's pretty short\"\n  import A\n  alias C.C\n  alias D.D\n\n  require C\n  require B\n\n  use A\n\n  alias C.C\n  alias A.A\n\n  @moduledoc \"README.md\"\n             |> File.read!()\n             |> String.split(\"<!-- MDOC !-->\")\n             |> Enum.fetch!(1)\nend\n```","ref":"module_directives.html#before"},{"type":"extras","title":"After - Module Directives (use, alias, ...)","doc":"```elixir\ndefmodule Foo do\n  @shortdoc \"it's pretty short\"\n  @moduledoc \"README.md\"\n             |> File.read!()\n             |> String.split(\"<!-- MDOC !-->\")\n             |> Enum.fetch!(1)\n  @behaviour Chaotic\n  @behaviour Lawful\n\n  use B\n  use A.A\n\n  import A.A\n  import C\n\n  alias A.A\n  alias C.C\n  alias D.D\n\n  require A\n  require B\n  require C\n\n  def c(x), do: y\n\n  @doc \"d doc\"\n  def d do\n    import Ecto.Query\n\n    alias H.H\n    alias X.X\n    alias Z.Z\n\n    X.foo()\n  end\nend\n```\n\nIf any line previously relied on an alias, the alias is fully expanded when it is moved above the alias:\n\n```elixir\n# Given\nalias Foo.Bar\nimport Bar\n# Styled\nimport Foo.Bar\n\nalias Foo.Bar\n```","ref":"module_directives.html#after"},{"type":"extras","title":"Alias Lifting - Module Directives (use, alias, ...)","doc":"When a module with three parts is referenced two or more times, styler creates a new alias for that module and uses it.\n\n```elixir\n# Given\nrequire A.B.C\n\nA.B.C.foo()\nA.B.C.bar()\n\n# Styled\nalias A.B.C\n\nrequire C\n\nC.foo()\nC.bar()\n```","ref":"module_directives.html#alias-lifting"},{"type":"extras","title":"Collisions - Module Directives (use, alias, ...)","doc":"Styler won't lift aliases that will collide with existing aliases, and likewise won't lift any module whose name would collide with a standard library name.\n\nYou can specify additional modules to exclude from lifting via the `:alias_lifting_exclude` configuration option. For the example above, the following configuration would keep Styler from creating the `alias A.B.C` node:\n\n```elixir\n# .formatter.exs\n[\n  plugins: [Styler],\n  styler: [alias_lifting_exclude: [:C]],\n]\n```","ref":"module_directives.html#collisions"},{"type":"extras","title":"Styler & Credo","doc":"","ref":"credo.html"},{"type":"extras","title":"Credo Rules Styler Replaces - Styler & Credo","doc":"If you're using Credo and Styler, **we recommend disabling these rules in `.credo.exs`** to save on unnecessary checks in CI.\nAs long as you're running `mix format --check-formatted` in CI, Styler will be enforcing the rules for you, so checking them with Credo is redundant.\n\nDisabling the rules means updating your `.credo.exs` depending on your configuration:\n\n- if you're using `checks: %{enabled: [...]}`, ensure none of the checks are listed in your enabled checks\n- if you're using `checks: %{disabled: [...]}`, copy/paste the snippet below into the list\n- if you're using `checks: [...]`, copy/paste the snippet below into the list and ensure none of the checks appear earlier in the list\n\n```elixir\n# Styler Rewrites\n#\n# The following rules are automatically rewritten by Styler and so disabled here to save time\n# Some of the rules have `priority: :high`, meaning Credo runs them unless we explicitly disable them\n# (removing them from this file wouldn't be enough, the `false` is required)\n#\n{Credo.Check.Consistency.MultiAliasImportRequireUse, false},\n{Credo.Check.Consistency.ParameterPatternMatching, false},\n{Credo.Check.Design.AliasUsage, false},\n{Credo.Check.Readability.AliasOrder, false},\n{Credo.Check.Readability.BlockPipe, false},\n{Credo.Check.Readability.LargeNumbers, false},\n{Credo.Check.Readability.ModuleDoc, false},\n{Credo.Check.Readability.MultiAlias, false},\n{Credo.Check.Readability.OneArityFunctionInPipe, false},\n{Credo.Check.Readability.ParenthesesOnZeroArityDefs, false},\n{Credo.Check.Readability.PipeIntoAnonymousFunctions, false},\n{Credo.Check.Readability.PreferImplicitTry, false},\n{Credo.Check.Readability.SinglePipe, false},\n{Credo.Check.Readability.StrictModuleLayout, false},\n{Credo.Check.Readability.StringSigils, false},\n{Credo.Check.Readability.UnnecessaryAliasExpansion, false},\n{Credo.Check.Readability.WithSingleClause, false},\n{Credo.Check.Refactor.CaseTrivialMatches, false},\n{Credo.Check.Refactor.CondStatements, false},\n{Credo.Check.Refactor.FilterCount, false},\n{Credo.Check.Refactor.MapInto, false},\n{Credo.Check.Refactor.MapJoin, false},\n{Credo.Check.Refactor.NegatedConditionsInUnless, false},\n{Credo.Check.Refactor.NegatedConditionsWithElse, false},\n{Credo.Check.Refactor.PipeChainStart, false},\n{Credo.Check.Refactor.RedundantWithClauseResult, false},\n{Credo.Check.Refactor.UnlessWithElse, false},\n{Credo.Check.Refactor.WithClauses, false},\n ```","ref":"credo.html#credo-rules-styler-replaces"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}